@@ -12,6 +12,7 @@ from typing import (
     Callable,
     ClassVar,
     Iterable,
+    Literal,
     Type,
     TypeVar,
     cast,
@@ -20,12 +21,12 @@ from typing import (
 
 from ._dataclass_utils import iter_fields
 from ._group import SignalGroup
-from ._signal import Signal
+from ._signal import Signal, SignalInstance
 
 if TYPE_CHECKING:
-    from typing_extensions import Literal
+    from _weakref import ref as ref
 
-    from ._signal import SignalInstance
+    from psygnal._weak_callback import RefErrorChoice, WeakCallback
 
 
 __all__ = ["is_evented", "get_evented_namespace", "SignalGroupDescriptor"]
@@ -125,6 +126,21 @@ def _pick_equality_operator(type_: type | None) -> EqOperator:
     return operator.eq
 
 
+class _DataclassFieldSignalInstance(SignalInstance):
+    def connect_setattr(
+        self,
+        obj: ref | object,
+        attr: str,
+        maxargs: int | None | object = 1,
+        *,
+        on_ref_error: RefErrorChoice = "warn",
+        priority: int = 0,
+    ) -> WeakCallback[None]:
+        return super().connect_setattr(
+            obj, attr, maxargs, on_ref_error=on_ref_error, priority=priority
+        )
+
+
 @lru_cache(maxsize=None)
 def _build_dataclass_signal_group(
     cls: type, equality_operators: Iterable[tuple[str, EqOperator]] | None = None
@@ -133,6 +149,7 @@ def _build_dataclass_signal_group(
     _equality_operators = dict(equality_operators) if equality_operators else {}
     signals = {}
     eq_map = _get_eq_operator_map(cls)
+    # create a Signal for each field in the dataclass
     for name, type_ in iter_fields(cls):
         if name in _equality_operators:
             if not callable(_equality_operators[name]):  # pragma: no cover
@@ -140,7 +157,10 @@ def _build_dataclass_signal_group(
             eq_map[name] = _equality_operators[name]
         else:
             eq_map[name] = _pick_equality_operator(type_)
-        signals[name] = Signal(object if type_ is None else type_)
+        field_type = object if type_ is None else type_
+        signals[name] = sig = Signal(field_type, field_type)
+        # patch in our custom SignalInstance class with maxargs=1 on connect_setattr
+        sig._signal_instance_class = _DataclassFieldSignalInstance
 
     return type(f"{cls.__name__}SignalGroup", (SignalGroup,), signals)
 
@@ -165,12 +185,14 @@ def get_evented_namespace(obj: object) -> str | None:
     ```python
     from psygnal import evented, get_evented_namespace, is_evented
 
+
     @evented(events_namespace="my_events")
-    class Foo:
-        ...
+    class Foo: ...
+
 
     assert get_evented_namespace(Foo) == "my_events"
     assert is_evented(Foo)
+    ```
     """
     return getattr(obj, PSYGNAL_GROUP_NAME, None)
 
@@ -187,22 +209,20 @@ class _changes_emitted:
     def __exit__(self, *args: Any) -> None:
         new: Any = getattr(self.obj, self.field, _NULL)
         if not _check_field_equality(type(self.obj), self.field, self._prev, new):
-            self.signal.emit(new)
+            self.signal.emit(new, self._prev)
 
 
 SetAttr = Callable[[Any, str, Any], None]
 
 
 @overload
-def evented_setattr(signal_group_name: str, super_setattr: SetAttr) -> SetAttr:
-    ...
+def evented_setattr(signal_group_name: str, super_setattr: SetAttr) -> SetAttr: ...
 
 
 @overload
 def evented_setattr(
     signal_group_name: str, super_setattr: Literal[None] | None = None
-) -> Callable[[SetAttr], SetAttr]:
-    ...
+) -> Callable[[SetAttr], SetAttr]: ...
 
 
 def evented_setattr(
@@ -224,6 +244,7 @@ def evented_setattr(
         class SignalInstanceProtocol(Protocol):
             def emit(self, *args: Any) -> Any: ...
 
+
         class SignalGroupProtocol(Protocol):
             def __getattr__(self, name: str) -> SignalInstanceProtocol: ...
         ```
@@ -248,9 +269,12 @@ def evented_setattr(
                 return super_setattr(self, name, value)
 
             group: SignalGroup | None = getattr(self, signal_group_name, None)
-            signal: SignalInstance | None = getattr(group, name, None)
+            if not isinstance(group, SignalGroup) or name not in group:
+                return super_setattr(self, name, value)
+
             # don't emit if the signal doesn't exist or has no listeners
-            if group is None or signal is None or len(signal) < 2 and not len(group):
+            signal: SignalInstance = group[name]
+            if len(signal) < 1:
                 return super_setattr(self, name, value)
 
             with _changes_emitted(self, name, signal):
@@ -288,6 +312,7 @@ class SignalGroupDescriptor:
         from dataclasses import dataclass
         from typing import ClassVar
 
+
         @dataclass
         class Foo:
             x: int
@@ -335,13 +360,15 @@ class SignalGroupDescriptor:
     from dataclasses import dataclass
     from psygnal import SignalGroupDescriptor
 
+
     @dataclass
     class Person:
         name: str
         age: int = 0
         events: ClassVar[SignalGroupDescriptor] = SignalGroupDescriptor()
 
-    john = Person('John', 40)
+
+    john = Person("John", 40)
     john.events.age.connect(print)
     john.age += 1  # prints 41
     ```
@@ -380,7 +407,8 @@ class SignalGroupDescriptor:
         try:
             # assign a new __setattr__ method to the class
             owner.__setattr__ = evented_setattr(  # type: ignore
-                self._name, owner.__setattr__  # type: ignore
+                cast(str, self._name),
+                owner.__setattr__,  # type: ignore
             )
         except Exception as e:  # pragma: no cover
             # not sure what might cause this ... but it will have consequences
@@ -394,12 +422,10 @@ class SignalGroupDescriptor:
     _instance_map: ClassVar[dict[int, SignalGroup]] = {}
 
     @overload
-    def __get__(self, instance: None, owner: type) -> SignalGroupDescriptor:
-        ...
+    def __get__(self, instance: None, owner: type) -> SignalGroupDescriptor: ...
 
     @overload
-    def __get__(self, instance: object, owner: type) -> SignalGroup:
-        ...
+    def __get__(self, instance: object, owner: type) -> SignalGroup: ...
 
     def __get__(
         self, instance: object, owner: type
@@ -422,17 +448,14 @@ class SignalGroupDescriptor:
                     setattr(instance, self._name, self._instance_map[obj_id])
 
             # clean up the cache when the instance is deleted
-            with contextlib.suppress(TypeError):
-                # on 3.7 this is type error, above it's not... but mypy yells about
-                # type ignore on 3.8+, so we do this funny business instead.
-                args = (instance, self._instance_map.pop, obj_id, None)
-                weakref.finalize(*args)  # type: ignore
+            with contextlib.suppress(TypeError):  # if it's not weakref-able
+                weakref.finalize(instance, self._instance_map.pop, obj_id, None)
 
         return self._instance_map[obj_id]
 
     def _create_group(self, owner: type) -> type[SignalGroup]:
         Group = self._signal_group or _build_dataclass_signal_group(owner, self._eqop)
-        if self._warn_on_no_fields and not Group._signals_:
+        if self._warn_on_no_fields and not Group._psygnal_signals:
             warnings.warn(
                 f"No mutable fields found on class {owner}: no events will be "
                 "emitted. (Is this a dataclass, attrs, msgspec, or pydantic model?)",
