@@ -1,6 +1,7 @@
 from __future__ import annotations
 
 import inspect
+import sys
 import threading
 import warnings
 import weakref
@@ -15,19 +16,20 @@ from typing import (
     ContextManager,
     Iterable,
     Iterator,
+    Literal,
     NoReturn,
     Type,
     TypeVar,
     Union,
     cast,
+    get_args,
+    get_origin,
     get_type_hints,
     overload,
 )
 
-from mypy_extensions import mypyc_attr
-from typing_extensions import get_args, get_origin
-
 from ._exceptions import EmitLoopError
+from ._mypyc import mypyc_attr
 from ._queue import QueuedCallback
 from ._weak_callback import (
     StrongFunction,
@@ -38,16 +40,20 @@ from ._weak_callback import (
 )
 
 if TYPE_CHECKING:
-    from typing_extensions import Literal
-
     from ._group import EmissionInfo
     from ._weak_callback import RefErrorChoice
 
-    ReducerFunc = Callable[[tuple, tuple], tuple]
+    # single function that does all the work of reducing an iterable of args
+    # to a single args
+    ReducerOneArg = Callable[[Iterable[tuple]], tuple]
+    # function that takes two args tuples. it will be passed to itertools.reduce
+    ReducerTwoArgs = Callable[[tuple, tuple], tuple]
+    ReducerFunc = Union[ReducerOneArg, ReducerTwoArgs]
 
 __all__ = ["Signal", "SignalInstance", "_compiled"]
 _NULL = object()
 F = TypeVar("F", bound=Callable)
+RECURSION_LIMIT = sys.getrecursionlimit()
 
 
 class Signal:
@@ -61,12 +67,15 @@ class Signal:
     ```python
     from psygnal import Signal
 
+
     class MyEmitter:
         changed = Signal(int)
 
+
     def receiver(arg: int):
         print("new value:", arg)
 
+
     emitter = MyEmitter()
     emitter.changed.connect(receiver)
     emitter.changed.emit(1)  # prints 'new value: 1'
@@ -116,6 +125,8 @@ class Signal:
         self.description = description
         self._check_nargs_on_connect = check_nargs_on_connect
         self._check_types_on_connect = check_types_on_connect
+        self._signal_instance_class: type[SignalInstance] = SignalInstance
+        self._signal_instance_cache: dict[int, SignalInstance] = {}
 
         if types and isinstance(types[0], Signature):
             self._signature = types[0]
@@ -139,12 +150,12 @@ class Signal:
             self._name = name
 
     @overload
-    def __get__(self, instance: None, owner: type[Any] | None = None) -> Signal:
-        ...  # pragma: no cover
+    def __get__(self, instance: None, owner: type[Any] | None = None) -> Signal: ...
 
     @overload
-    def __get__(self, instance: Any, owner: type[Any] | None = None) -> SignalInstance:
-        ...  # pragma: no cover
+    def __get__(
+        self, instance: Any, owner: type[Any] | None = None
+    ) -> SignalInstance: ...
 
     def __get__(
         self, instance: Any, owner: type[Any] | None = None
@@ -170,23 +181,46 @@ class Signal:
         """
         if instance is None:
             return self
-        name = cast("str", self._name)
-        signal_instance = SignalInstance(
+        if id(instance) in self._signal_instance_cache:
+            return self._signal_instance_cache[id(instance)]
+        signal_instance = self._create_signal_instance(instance)
+
+        # cache this signal instance so that future access returns the same instance.
+        try:
+            # first, try to assign it to instance.name ... this essentially breaks the
+            # descriptor, (i.e. __get__ will never again be called for this instance)
+            # (note, this is the same mechanism used in the `cached_property` decorator)
+            setattr(instance, cast("str", self._name), signal_instance)
+        except AttributeError:
+            # if that fails, which may happen in slotted classes, then we fall back to
+            # our internal cache
+            self._cache_signal_instance(instance, signal_instance)
+
+        return signal_instance
+
+    def _cache_signal_instance(
+        self, instance: Any, signal_instance: SignalInstance
+    ) -> None:
+        """Cache a signal instance on the instance."""
+        # fallback signal instance cache as last resort.  We use the object id
+        # instead a WeakKeyDictionary because we can't guarantee that the instance
+        # is hashable or weak-referenceable.  and we use a finalize to remove the
+        # cache when the instance is destroyed (if the object is weak-referenceable).
+        obj_id = id(instance)
+        self._signal_instance_cache[obj_id] = signal_instance
+        with suppress(TypeError):
+            weakref.finalize(instance, self._signal_instance_cache.pop, obj_id, None)
+
+    def _create_signal_instance(
+        self, instance: Any, name: str | None = None
+    ) -> SignalInstance:
+        return self._signal_instance_class(
             self.signature,
             instance=instance,
-            name=name,
+            name=name or self._name,
             check_nargs_on_connect=self._check_nargs_on_connect,
             check_types_on_connect=self._check_types_on_connect,
         )
-        # instead of caching this signal instance on self, we just assign it
-        # to instance.name ... this essentially breaks the descriptor,
-        # (i.e. __get__ will never again be called for this instance, and we have no
-        # idea how many instances are out there),
-        # but it allows us to prevent creating a key for this instance (which may
-        # not be hashable or weak-referenceable), and also provides a significant
-        # speedup on attribute access (affecting everything).
-        setattr(instance, name, signal_instance)
-        return signal_instance
 
     @classmethod
     @contextmanager
@@ -209,6 +243,7 @@ class Signal:
         ```python
         from psygnal import Signal
 
+
         def my_callback():
             source = Signal.current_emitter()
         ```
@@ -245,6 +280,7 @@ class SignalInstance:
     class Emitter:
         signal = Signal()
 
+
     e = Emitter()
 
     # when accessed on an *instance* of Emitter,
@@ -297,7 +333,7 @@ class SignalInstance:
     ) -> None:
         self._name = name
         self._instance: Callable = self._instance_ref(instance)
-        self._args_queue: list[Any] = []  # filled when paused
+        self._args_queue: list[tuple] = []  # filled when paused
 
         if isinstance(signature, (list, tuple)):
             signature = _build_signature(*signature)
@@ -314,6 +350,9 @@ class SignalInstance:
         self._is_blocked: bool = False
         self._is_paused: bool = False
         self._lock = threading.RLock()
+        self._emit_queue: list[tuple] = []
+        # whether any slots in self._slots have a priority other than 0
+        self._priority_in_use = False
 
     @staticmethod
     def _instance_ref(instance: Any) -> Callable[[], Any]:
@@ -357,8 +396,8 @@ class SignalInstance:
         unique: bool | str = ...,
         max_args: int | None = None,
         on_ref_error: RefErrorChoice = ...,
-    ) -> Callable[[F], F]:
-        ...  # pragma: no cover
+        priority: int = ...,
+    ) -> Callable[[F], F]: ...
 
     @overload
     def connect(
@@ -371,8 +410,8 @@ class SignalInstance:
         unique: bool | str = ...,
         max_args: int | None = None,
         on_ref_error: RefErrorChoice = ...,
-    ) -> F:
-        ...  # pragma: no cover
+        priority: int = ...,
+    ) -> F: ...
 
     def connect(
         self,
@@ -384,6 +423,7 @@ class SignalInstance:
         unique: bool | str = False,
         max_args: int | None = None,
         on_ref_error: RefErrorChoice = "warn",
+        priority: int = 0,
     ) -> Callable[[F], F] | F:
         """Connect a callback (`slot`) to this signal.
 
@@ -400,8 +440,7 @@ class SignalInstance:
 
         ```python
         @signal.connect
-        def my_function():
-            ...
+        def my_function(): ...
         ```
 
         !!!important
@@ -448,6 +487,11 @@ class SignalInstance:
             ReferenceError will be raised.  If 'warn' (default), a warning will be
             issued and a strong-reference will be used. If 'ignore' a strong-reference
             will be used (silently).
+        priority : int
+            The priority of the callback. This is used to determine the order in which
+            callbacks are called when multiple are connected to the same signal.
+            Higher priority callbacks are called first. Negative values are allowed.
+            The default is 0.
 
         Raises
         ------
@@ -492,24 +536,54 @@ class SignalInstance:
                         extra = f"- Slot types {slot_sig} do not match types in signal."
                         self._raise_connection_error(slot, extra)
 
-                # this type ignore could be fixed with ParamSpec, but that's not yet
-                # supported by mypyc.  we need cb to be a WeakCallback[R], but we can't
-                # preserve the full typing information of the callback without using
-                # Callable[ParamSpec, R] or a general TypeVar('F', bound=Callable).
-                cb = weak_callback(  # type: ignore [var-annotated]
+                cb = weak_callback(
                     slot,
                     max_args=max_args,
                     finalize=self._try_discard,
                     on_ref_error=_on_ref_err,
+                    priority=priority,
                 )
-                if thread is None:
-                    self._slots.append(cb)
-                else:
-                    self._slots.append(QueuedCallback(cb, thread=thread))
+                if thread is not None:
+                    cb = QueuedCallback(cb, thread=thread)
+                self._append_slot(cb)
             return slot
 
         return _wrapper if slot is None else _wrapper(slot)
 
+    def _append_slot(self, slot: WeakCallback) -> None:
+        """Append a slot to the list of slots.
+
+        Implementing this as a method allows us to override/extend it in subclasses.
+        """
+        # if no previously connected slots have a priority, and this slot also
+        # has no priority, we can just (quickly) append it to the end of the list.
+        if not self._priority_in_use:
+            if not slot.priority:
+                self._slots.append(slot)
+                return
+            # remember that we have a priority in use, so we skip this check
+            self._priority_in_use = True
+
+        # otherwise we need to (slowly) iterate over self._slots to
+        # insert the slot in the correct position based on priority.
+        # High priority slots are placed at the front of the list
+        # low/negative priority slots are at the end of the list
+        for i, s in enumerate(self._slots):
+            if s.priority < slot.priority:
+                self._slots.insert(i, slot)
+                return
+        self._slots.append(slot)
+
+    def _remove_slot(self, slot: Literal["all"] | int | WeakCallback) -> None:
+        """Remove a slot from the list of slots."""
+        # implementing this as a method allows us to override/extend it in subclasses
+        if slot == "all":
+            self._slots.clear()
+        elif isinstance(slot, int):
+            self._slots.pop(slot)
+        else:
+            self._slots.remove(cast("WeakCallback", slot))
+
     def _try_discard(self, callback: WeakCallback, missing_ok: bool = True) -> None:
         """Try to discard a callback from the list of slots.
 
@@ -521,18 +595,19 @@ class SignalInstance:
             If `True`, do not raise an error if the callback is not found in the list.
         """
         try:
-            self._slots.remove(callback)
+            self._remove_slot(callback)
         except ValueError:
             if not missing_ok:
                 raise
 
     def connect_setattr(
         self,
-        obj: weakref.ref | object,
+        obj: object,
         attr: str,
-        maxargs: int | None = None,
+        maxargs: int | None | object = _NULL,
         *,
         on_ref_error: RefErrorChoice = "warn",
+        priority: int = 0,
     ) -> WeakCallback[None]:
         """Bind an object attribute to the emitted value of this signal.
 
@@ -544,8 +619,8 @@ class SignalInstance:
 
         Parameters
         ----------
-        obj : Union[weakref.ref, object]
-            An object or weak reference (deprecated) to an object.
+        obj : object
+            An object.
         attr : str
             The name of an attribute on `obj` that should be set to the value of this
             signal when emitted.
@@ -556,6 +631,11 @@ class SignalInstance:
             ReferenceError will be raised.  If 'warn' (default), a warning will be
             issued and a strong-reference will be used. If 'ignore' a strong-reference
             will be used (silently).
+        priority : int
+            The priority of the callback. This is used to determine the order in which
+            callbacks are called when multiple are connected to the same signal.
+            Higher priority callbacks are called first. Negative values are allowed.
+            The default is 0.
 
         Returns
         -------
@@ -574,25 +654,24 @@ class SignalInstance:
         --------
         >>> class T:
         ...     sig = Signal(int)
-        ...
         >>> class SomeObj:
         ...     x = 1
-        ...
         >>> t = T()
         >>> my_obj = SomeObj()
-        >>> t.sig.connect_setattr(my_obj, 'x')
+        >>> t.sig.connect_setattr(my_obj, "x")
         >>> t.sig.emit(5)
         >>> assert my_obj.x == 5
         """
-        if isinstance(obj, weakref.ReferenceType):  # pragma: no cover
+        if maxargs is _NULL:
             warnings.warn(
-                'Using a weakref as the "obj" argument is deprecated. '
-                "Use the object directly instead. This will raise an error in "
-                "a future release.",
+                "The default value of maxargs will change from `None` to `1` in "
+                "version 0.11. To silence this warning, provide an explicit value for "
+                "maxargs (`None` for current behavior, `1` for future behavior).",
                 FutureWarning,
                 stacklevel=2,
             )
-            obj = obj()
+            maxargs = None
+
         if not hasattr(obj, attr):
             raise AttributeError(f"Object {obj} has no attribute {attr!r}")
 
@@ -600,11 +679,12 @@ class SignalInstance:
             caller = WeakSetattr(
                 obj,
                 attr,
-                max_args=maxargs,
+                max_args=cast("int | None", maxargs),
                 finalize=self._try_discard,
                 on_ref_error=on_ref_error,
+                priority=priority,
             )
-            self._slots.append(caller)
+            self._append_slot(caller)
         return caller
 
     def disconnect_setattr(
@@ -635,11 +715,12 @@ class SignalInstance:
 
     def connect_setitem(
         self,
-        obj: weakref.ref | object,
+        obj: object,
         key: str,
-        maxargs: int | None = None,
+        maxargs: int | None | object = _NULL,
         *,
         on_ref_error: RefErrorChoice = "warn",
+        priority: int = 0,
     ) -> WeakCallback[None]:
         """Bind a container item (such as a dict key) to emitted value of this signal.
 
@@ -651,8 +732,8 @@ class SignalInstance:
 
         Parameters
         ----------
-        obj : Union[weakref.ref, object]
-            An object or weak reference (deprecated) to an object.
+        obj : object
+            An object.
         key : str
             Name of the key in `obj` that should be set to the value of this
             signal when emitted
@@ -663,6 +744,11 @@ class SignalInstance:
             ReferenceError will be raised.  If 'warn' (default), a warning will be
             issued and a strong-reference will be used. If 'ignore' a strong-reference
             will be used (silently).
+        priority : int
+            The priority of the callback. This is used to determine the order in which
+            callbacks are called when multiple are connected to the same signal.
+            Higher priority callbacks are called first. Negative values are allowed.
+            The default is 0.
 
         Returns
         -------
@@ -681,34 +767,35 @@ class SignalInstance:
         --------
         >>> class T:
         ...     sig = Signal(int)
-        ...
         >>> t = T()
         >>> my_obj = dict()
-        >>> t.sig.connect_setitem(my_obj, 'x')
+        >>> t.sig.connect_setitem(my_obj, "x")
         >>> t.sig.emit(5)
-        >>> assert my_obj == {'x': 5}
+        >>> assert my_obj == {"x": 5}
         """
-        if isinstance(obj, weakref.ReferenceType):  # pragma: no cover
+        if maxargs is _NULL:
             warnings.warn(
-                'Using a weakref as the "obj" argument is deprecated. '
-                "Use the object directly instead. This will raise an error in "
-                "a future release.",
+                "The default value of maxargs will change from `None` to `1` in"
+                "version 0.11. To silence this warning, provide an explicit value for "
+                "maxargs (`None` for current behavior, `1` for future behavior).",
                 FutureWarning,
                 stacklevel=2,
             )
-            obj = obj()
+            maxargs = None
+
         if not hasattr(obj, "__setitem__"):
             raise TypeError(f"Object {obj} does not support __setitem__")
 
         with self._lock:
             caller = WeakSetitem(
-                obj,  # type: ignore
+                obj,
                 key,
-                max_args=maxargs,
+                max_args=cast("int | None", maxargs),
                 finalize=self._try_discard,
                 on_ref_error=on_ref_error,
+                priority=priority,
             )
-            self._slots.append(caller)
+            self._append_slot(caller)
 
         return caller
 
@@ -775,9 +862,8 @@ class SignalInstance:
                 ) from e
             raise
 
-        n_spec_params = len(spec.parameters)
         # if `slot` requires more arguments than we will provide, raise.
-        if minargs > n_spec_params:
+        if minargs > (n_spec_params := len(spec.parameters)):
             extra = (
                 f"- Slot requires at least {minargs} positional "
                 f"arguments, but spec only provides {n_spec_params}"
@@ -821,12 +907,12 @@ class SignalInstance:
         with self._lock:
             if slot is None:
                 # NOTE: clearing an empty list is actually a RuntimeError in Qt
-                self._slots.clear()
+                self._remove_slot("all")
                 return
 
             idx = self._slot_index(slot)
             if idx != -1:
-                self._slots.pop(idx)
+                self._remove_slot(idx)
             elif not missing_ok:
                 raise ValueError(f"slot is not connected: {slot}")
 
@@ -838,34 +924,9 @@ class SignalInstance:
         """Return number of connected slots."""
         return len(self._slots)
 
-    @overload
     def emit(
-        self,
-        *args: Any,
-        check_nargs: bool = False,
-        check_types: bool = False,
-        asynchronous: Literal[False] = False,
+        self, *args: Any, check_nargs: bool = False, check_types: bool = False
     ) -> None:
-        ...  # pragma: no cover
-
-    @overload
-    def emit(
-        self,
-        *args: Any,
-        check_nargs: bool = False,
-        check_types: bool = False,
-        asynchronous: Literal[True],
-    ) -> EmitThread | None:
-        # will return `None` if emitter is blocked
-        ...  # pragma: no cover
-
-    def emit(
-        self,
-        *args: Any,
-        check_nargs: bool = False,
-        check_types: bool = False,
-        asynchronous: bool = False,
-    ) -> EmitThread | None:
         """Emit this signal with arguments `args`.
 
         !!! note
@@ -885,13 +946,6 @@ class SignalInstance:
             If `False` and the provided arguments do not match the types declared by
             the signature of this Signal, raise `TypeError`.  Incurs some overhead.
             by default False.
-        asynchronous : bool
-            If `True`, run signal emission in another thread. by default `False`.
-            **DEPRECATED:**. *If you need to emit from a thread, please just create
-            your own [`threading.Thread`][] and call
-            [`SignalInstance.emit`][psygnal.SignalInstance.emit]. See also the `thread`
-            parameter in the [`SignalInstance.connect`][psygnal.SignalInstance.connect]
-            method.*
 
         Raises
         ------
@@ -929,69 +983,45 @@ class SignalInstance:
 
             SignalInstance._debug_hook(EmissionInfo(self, args))
 
-        if asynchronous:
-            warnings.warn(
-                "The `asynchronous` parameter is deprecated and will be removed in a "
-                "future release. If you need this, please create your own "
-                "`threading.Thread` and call `SignalInstance.emit`. See also the new "
-                "`thread` parameter in the `SignalInstance.connect` method.",
-                FutureWarning,
-                stacklevel=2,
-            )
-            sd = EmitThread(self, args)
-            sd.start()
-            return sd
-
         self._run_emit_loop(args)
         return None
 
-    @overload
     def __call__(
-        self,
-        *args: Any,
-        check_nargs: bool = False,
-        check_types: bool = False,
-        asynchronous: Literal[False] = False,
+        self, *args: Any, check_nargs: bool = False, check_types: bool = False
     ) -> None:
-        ...  # pragma: no cover
-
-    @overload
-    def __call__(
-        self,
-        *args: Any,
-        check_nargs: bool = False,
-        check_types: bool = False,
-        asynchronous: Literal[True],
-    ) -> EmitThread | None:
-        # will return `None` if emitter is blocked
-        ...  # pragma: no cover
-
-    def __call__(
-        self,
-        *args: Any,
-        check_nargs: bool = False,
-        check_types: bool = False,
-        asynchronous: bool = False,
-    ) -> EmitThread | None:
         """Alias for `emit()`."""
-        return self.emit(  # type: ignore
+        return self.emit(
             *args,
             check_nargs=check_nargs,
             check_types=check_types,
-            asynchronous=asynchronous,
         )
 
     def _run_emit_loop(self, args: tuple[Any, ...]) -> None:
         # allow receiver to query sender with Signal.current_emitter()
         with self._lock:
-            with Signal._emitting(self):
-                for caller in self._slots:
-                    try:
-                        caller.cb(args)
-                    except Exception as e:
-                        raise EmitLoopError(
-                            cb=caller, args=args, exc=e, signal=self
-                        ) from e
+            self._emit_queue.append(args)
+
+            if len(self._emit_queue) > 1:
+                return None
+            try:
+                with Signal._emitting(self):
+                    i = 0
+                    while i < len(self._emit_queue):
+                        args = self._emit_queue[i]
+                        for caller in self._slots:
+                            caller.cb(args)
+                            if len(self._emit_queue) > RECURSION_LIMIT:
+                                raise RecursionError
+                        i += 1
+            except RecursionError as e:
+                raise RecursionError(
+                    f"RecursionError in {caller.slot_repr()} when "
+                    f"emitting signal {self.name!r} with args {args}"
+                ) from e
+            except Exception as e:
+                raise EmitLoopError(cb=caller, args=args, exc=e, signal=self) from e
+            finally:
+                self._emit_queue.clear()
 
         return None
 
@@ -1043,15 +1073,20 @@ class SignalInstance:
 
         Parameters
         ----------
-        reducer : Callable[[tuple, tuple], Any], optional
-            If provided, all gathered args will be reduced into a single argument by
-            passing `reducer` to `functools.reduce`.
-            NOTE: args passed to `emit` are collected as tuples, so the two arguments
-            passed to `reducer` will always be tuples. `reducer` must handle that and
-            return an args tuple.
-            For example, three `emit(1)` events would be reduced and re-emitted as
-            follows: `self.emit(*functools.reduce(reducer, [(1,), (1,), (1,)]))`
+        reducer : Callable | None
+            A optional function to reduce the args collected while paused into a single
+            emitted group of args.  If not provided, all emissions will be re-emitted
+            as they were collected when the signal is resumed. May be:
 
+            - a function that takes two args tuples and returns a single args tuple.
+              This will be passed to `functools.reduce` and is expected to reduce all
+              collected/emitted args into a single tuple.
+              For example, three `emit(1)` events would be reduced and re-emitted as
+              follows: `self.emit(*functools.reduce(reducer, [(1,), (1,), (1,)]))`
+            - a function that takes a single argument (an iterable of args tuples) and
+              returns a tuple (the reduced args). This will be *not* be passed to
+              `functools.reduce`. If `reducer` is a function that takes a single
+              argument, `initial` will be ignored.
         initial: any, optional
             initial value to pass to `functools.reduce`
 
@@ -1072,11 +1107,19 @@ class SignalInstance:
         # EventedModel.update, it may be undefined (as seen in tests)
         if not getattr(self, "_args_queue", None):
             return
+        if len(self._slots) == 0:
+            self._args_queue.clear()
+            return
+
         if reducer is not None:
-            if initial is _NULL:
-                args = reduce(reducer, self._args_queue)
+            if len(inspect.signature(reducer).parameters) == 1:
+                args = cast("ReducerOneArg", reducer)(self._args_queue)
             else:
-                args = reduce(reducer, self._args_queue, initial)
+                reducer = cast("ReducerTwoArgs", reducer)
+                if initial is _NULL:
+                    args = reduce(reducer, self._args_queue)
+                else:
+                    args = reduce(reducer, self._args_queue, initial)
             self._run_emit_loop(args)
         else:
             for args in self._args_queue:
@@ -1090,14 +1133,20 @@ class SignalInstance:
 
         Parameters
         ----------
-        reducer : Callable[[tuple, tuple], Any], optional
-            If provided, all gathered args will be reduced into a single argument by
-            passing `reducer` to `functools.reduce`.
-            NOTE: args passed to `emit` are collected as tuples, so the two arguments
-            passed to `reducer` will always be tuples. `reducer` must handle that and
-            return an args tuple.
-            For example, three `emit(1)` events would be reduced and re-emitted as
-            follows: `self.emit(*functools.reduce(reducer, [(1,), (1,), (1,)]))`
+        reducer : Callable | None
+            A optional function to reduce the args collected while paused into a single
+            emitted group of args.  If not provided, all emissions will be re-emitted
+            as they were collected when the signal is resumed. May be:
+
+            - a function that takes two args tuples and returns a single args tuple.
+              This will be passed to `functools.reduce` and is expected to reduce all
+              collected/emitted args into a single tuple.
+              For example, three `emit(1)` events would be reduced and re-emitted as
+              follows: `self.emit(*functools.reduce(reducer, [(1,), (1,), (1,)]))`
+            - a function that takes a single argument (an iterable of args tuples) and
+              returns a tuple (the reduced args). This will be *not* be passed to
+              `functools.reduce`. If `reducer` is a function that takes a single
+              argument, `initial` will be ignored.
         initial: any, optional
             initial value to pass to `functools.reduce`
 
@@ -1121,6 +1170,8 @@ class SignalInstance:
             "_args_queue",
             "_check_nargs_on_connect",
             "_check_types_on_connect",
+            "_emit_queue",
+            "_priority_in_use",
         )
         dd = {slot: getattr(self, slot) for slot in attrs}
         dd["_instance"] = self._instance()
@@ -1182,21 +1233,6 @@ class _SignalPauser:
             self._signal.resume(self._reducer, self._initial)
 
 
-class EmitThread(threading.Thread):
-    """A thread to emit a signal asynchronously."""
-
-    def __init__(self, signal_instance: SignalInstance, args: tuple[Any, ...]) -> None:
-        super().__init__(name=signal_instance.name)
-        self._signal_instance = signal_instance
-        self.args = args
-        # current = threading.currentThread()
-        # self.parent = (current.getName(), current.ident)
-
-    def run(self) -> None:
-        """Run thread."""
-        self._signal_instance._run_emit_loop(self.args)
-
-
 # #############################################################################
 # #############################################################################
 
