@@ -1,15 +1,27 @@
 from __future__ import annotations
 
+import inspect
 from itertools import chain
-from typing import TYPE_CHECKING, Any, Iterable, Iterator, MutableSet, TypeVar
+from typing import (
+    TYPE_CHECKING,
+    Any,
+    Callable,
+    Final,
+    Iterable,
+    Iterator,
+    Mapping,
+    MutableSet,
+    TypeVar,
+    get_args,
+)
 
 from psygnal import Signal, SignalGroup
 
 if TYPE_CHECKING:
-    from typing_extensions import Final
+    from typing import Self
+
 
 _T = TypeVar("_T")
-_Cls = TypeVar("_Cls", bound="_BaseMutableSet")
 
 
 class BailType:
@@ -72,14 +84,12 @@ class _BaseMutableSet(MutableSet[_T]):
     def _pre_add_hook(self, item: _T) -> _T | BailType:
         return item  # pragma: no cover
 
-    def _post_add_hook(self, item: _T) -> None:
-        ...  # pragma: no cover
+    def _post_add_hook(self, item: _T) -> None: ...
 
     def _pre_discard_hook(self, item: _T) -> _T | BailType:
         return item  # pragma: no cover
 
-    def _post_discard_hook(self, item: _T) -> None:
-        ...  # pragma: no cover
+    def _post_discard_hook(self, item: _T) -> None: ...
 
     def _do_add(self, item: _T) -> None:
         self._data.add(item)
@@ -89,15 +99,13 @@ class _BaseMutableSet(MutableSet[_T]):
 
     # -------- To match set API
 
-    def __copy__(self: _Cls) -> _Cls:
-        inst = self.__class__.__new__(self.__class__)
-        inst.__dict__.update(self.__dict__)
-        return inst
+    def __copy__(self) -> Self:
+        return self.copy()
 
-    def copy(self: _Cls) -> _Cls:
+    def copy(self) -> Self:
         return self.__class__(self)
 
-    def difference(self: _Cls, *s: Iterable[_T]) -> _Cls:
+    def difference(self, *s: Iterable[_T]) -> Self:
         """Return the difference of two or more sets as a new set.
 
         (i.e. all elements that are in this set but not the others.)
@@ -110,7 +118,7 @@ class _BaseMutableSet(MutableSet[_T]):
         for i in chain(*s):
             self.discard(i)
 
-    def intersection(self: _Cls, *s: Iterable[_T]) -> _Cls:
+    def intersection(self, *s: Iterable[_T]) -> Self:
         """Return the intersection of two sets as a new set.
 
         (i.e. all elements that are in both sets.)
@@ -133,7 +141,7 @@ class _BaseMutableSet(MutableSet[_T]):
         """Report whether this set contains another set."""
         return set(self).issuperset(__s)
 
-    def symmetric_difference(self: _Cls, __s: Iterable[_T]) -> _Cls:
+    def symmetric_difference(self, __s: Iterable[_T]) -> Self:
         """Return the symmetric difference of two sets as a new set.
 
         (i.e. all elements that are in exactly one of the sets.)
@@ -150,7 +158,7 @@ class _BaseMutableSet(MutableSet[_T]):
         for i in __s:
             self.discard(i) if i in self else self.add(i)
 
-    def union(self: _Cls, *s: Iterable[_T]) -> _Cls:
+    def union(self, *s: Iterable[_T]) -> Self:
         """Return the union of sets as a new set.
 
         (i.e. all elements that are in either set.)
@@ -159,6 +167,23 @@ class _BaseMutableSet(MutableSet[_T]):
         new.update(*s)
         return new
 
+    # PYDANTIC SUPPORT
+
+    @classmethod
+    def __get_pydantic_core_schema__(
+        cls, source_type: Any, handler: Callable
+    ) -> Mapping[str, Any]:
+        """Return the Pydantic core schema for this object."""
+        from pydantic_core import core_schema
+
+        args = get_args(source_type)
+        return core_schema.no_info_after_validator_function(
+            function=cls,
+            schema=core_schema.set_schema(
+                items_schema=handler(args[0]) if args else None,
+            ),
+        )
+
 
 class OrderedSet(_BaseMutableSet[_T]):
     """A set that preserves insertion order, uses dict behind the scenes."""
@@ -236,22 +261,22 @@ class EventedSet(_BaseMutableSet[_T]):
 
     def update(self, *others: Iterable[_T]) -> None:
         """Update this set with the union of this set and others."""
-        with self.events.items_changed.paused(_reduce_events, ((), ())):
+        with self.events.items_changed.paused(_reduce_events):
             super().update(*others)
 
     def clear(self) -> None:
         """Remove all elements from this set."""
-        with self.events.items_changed.paused(_reduce_events, ((), ())):
+        with self.events.items_changed.paused(_reduce_events):
             super().clear()
 
     def difference_update(self, *s: Iterable[_T]) -> None:
         """Remove all elements of another set from this set."""
-        with self.events.items_changed.paused(_reduce_events, ((), ())):
+        with self.events.items_changed.paused(_reduce_events):
             super().difference_update(*s)
 
     def intersection_update(self, *s: Iterable[_T]) -> None:
         """Update this set with the intersection of itself and another."""
-        with self.events.items_changed.paused(_reduce_events, ((), ())):
+        with self.events.items_changed.paused(_reduce_events):
             super().intersection_update(*s)
 
     def symmetric_difference_update(self, __s: Iterable[_T]) -> None:
@@ -303,8 +328,15 @@ class EventedOrderedSet(EventedSet, OrderedSet[_T]):
         super().__init__(iterable)
 
 
-def _reduce_events(a: tuple, b: tuple) -> tuple[tuple, tuple]:
-    """Combine two events (a and b) each of which contain (added, removed)."""
-    a0, a1 = a
-    b0, b1 = b
-    return (a0 + b0, a1 + b1)
+def _reduce_events(li: Iterable[tuple[Iterable, Iterable]]) -> tuple[tuple, tuple]:
+    """Combine multiple events into a single event."""
+    added_li: list = []
+    removed_li: list = []
+    for added, removed in li:
+        added_li.extend(added)
+        removed_li.extend(removed)
+    return tuple(added_li), tuple(removed_li)
+
+
+# for performance reasons
+_reduce_events.__signature__ = inspect.signature(_reduce_events)  # type: ignore [attr-defined]
