@@ -1,13 +1,14 @@
 """These utilities may help when using signals and evented objects."""
+
 from __future__ import annotations
 
-from contextlib import contextmanager
+from contextlib import contextmanager, suppress
 from functools import partial
 from pathlib import Path
 from typing import Any, Callable, Generator, Iterator
 from warnings import warn
 
-from ._group import EmissionInfo
+from ._group import EmissionInfo, SignalGroup, SignalRelay
 from ._signal import SignalInstance
 
 __all__ = ["monitor_events", "iter_signal_instances"]
@@ -58,6 +59,10 @@ def monitor_events(
             )
         disconnectors = set()
         for siginst in iter_signal_instances(obj, include_private_attrs):
+            if isinstance(siginst, SignalRelay):
+                # TODO: ... but why?
+                continue
+
             if _old_api:
 
                 def _report(*args: Any, signal: SignalInstance = siginst) -> None:
@@ -100,9 +105,12 @@ def iter_signal_instances(
     """
     for n in dir(obj):
         if include_private_attrs or not n.startswith("_"):
-            attr = getattr(obj, n)
-            if isinstance(attr, SignalInstance):
-                yield attr
+            with suppress(AttributeError, FutureWarning):
+                attr = getattr(obj, n)
+                if isinstance(attr, SignalInstance):
+                    yield attr
+                if isinstance(attr, SignalGroup):
+                    yield attr._psygnal_relay
 
 
 _COMPILED_EXTS = (".so", ".pyd")
