@@ -1,11 +1,10 @@
 import inspect
 import sys
-from typing import Any, ClassVar, List, Sequence, Union
+from typing import Any, ClassVar, List, Protocol, Sequence, Union, runtime_checkable
 from unittest.mock import Mock, call, patch
 
 import numpy as np
 import pytest
-from typing_extensions import Protocol, runtime_checkable
 
 try:
     from pydantic import PrivateAttr
@@ -15,7 +14,8 @@ except ImportError:
 import pydantic.version
 from pydantic import BaseModel
 
-from psygnal import EmissionInfo, EventedModel, SignalGroup
+from psygnal import EmissionInfo, EventedModel
+from psygnal._group import SignalGroup
 
 PYDANTIC_V2 = pydantic.version.VERSION.startswith("2")
 
@@ -69,11 +69,12 @@ def test_evented_model():
 
     # test event system
     assert isinstance(user.events, SignalGroup)
+    # with pytest.warns(FutureWarning):
     assert "id" in user.events.signals
     assert "name" in user.events.signals
 
     # ClassVars are excluded from events
-    assert "age" not in user.events.signals
+    assert "age" not in user.events
 
     id_mock = Mock()
     name_mock = Mock()
@@ -90,7 +91,6 @@ def test_evented_model():
     name_mock.assert_not_called()
 
 
-@pytest.mark.skipif(sys.version_info < (3, 8), reason="requires python3.8 or higher")
 def test_evented_model_array_updates():
     """Test updating an evented pydantic model with an array."""
 
@@ -175,7 +175,7 @@ def test_evented_model_da_array_equality():
     assert model1 == model2
 
 
-def test_values_updated():
+def test_values_updated() -> None:
     class User(EventedModel):
         """Demo evented model.
 
@@ -201,7 +201,12 @@ def test_values_updated():
     user1_events = Mock()
     u1_id_events = Mock()
     u2_id_events = Mock()
-    user1.events.connect(user1_events)
+
+    user1.events.all.connect(user1_events)
+    user1.events.all.connect(user1_events)
+
+    user1.events.id.connect(u1_id_events)
+    user2.events.id.connect(u2_id_events)
     user1.events.id.connect(u1_id_events)
     user2.events.id.connect(u2_id_events)
 
@@ -256,8 +261,7 @@ def test_update_with_inner_model_union():
 def test_update_with_inner_model_protocol():
     @runtime_checkable
     class InnerProtocol(Protocol):
-        def string(self) -> str:
-            ...
+        def string(self) -> str: ...
 
         # Protocol fields are not successfully set without explicit validation.
         @classmethod
@@ -522,7 +526,7 @@ def test_evented_model_with_property_setters():
 
 def test_evented_model_with_property_setters_events():
     t = T()
-    assert "c" in t.events.signals  # the setter has an event
+    assert "c" in t.events  # the setter has an event
     mock_a = Mock()
     mock_b = Mock()
     mock_c = Mock()
@@ -549,6 +553,9 @@ def test_evented_model_with_property_setters_events():
     mock_c.assert_called_with([5, 20])
     mock_b.assert_not_called()
     assert t.c == [5, 20]
+    mock_a.reset_mock()
+    t.a = 5  # no change, no events
+    mock_a.assert_not_called()
 
 
 def test_non_setter_with_dependencies() -> None:
@@ -560,12 +567,10 @@ def test_non_setter_with_dependencies() -> None:
             x: int
 
             @property
-            def y(self):
-                ...
+            def y(self): ...
 
             @y.setter
-            def y(self, v):
-                ...
+            def y(self, v): ...
 
             if PYDANTIC_V2:
                 model_config = {
@@ -586,12 +591,10 @@ def test_unrecognized_property_dependencies():
             x: int
 
             @property
-            def y(self):
-                ...
+            def y(self): ...
 
             @y.setter
-            def y(self, v):
-                ...
+            def y(self, v): ...
 
             if PYDANTIC_V2:
                 model_config = {
@@ -647,8 +650,7 @@ def test_setter_inheritance():
 
     assert M(x=2).x == 2
 
-    class N(M):
-        ...
+    class N(M): ...
 
     assert N(x=2).x == 2
 
@@ -837,7 +839,7 @@ def test_connect_only_to_events() -> None:
     check_mock.assert_not_called()
     mock1.assert_not_called()
 
-    m.events.connect(mock1)
+    m.events.all.connect(mock1)
     with patch.object(
         model_module,
         "_check_field_equality",
@@ -846,3 +848,52 @@ def test_connect_only_to_events() -> None:
         m.a = 3
     check_mock.assert_has_calls([call(Model, "a", 3, 1)])
     mock1.assert_called_once()
+
+
+def test_if_event_is_emitted_only_once() -> None:
+    """Check if, for complex property setters, the event is emitted only once."""
+
+    class SampleClass(EventedModel):
+        a: int = 1
+        b: int = 2
+
+        if PYDANTIC_V2:
+            model_config = {
+                "allow_property_setters": True,
+                "guess_property_dependencies": True,
+            }
+        else:
+
+            class Config:
+                allow_property_setters = True
+                guess_property_dependencies = True
+
+        @property
+        def c(self):
+            return self.a + self.b
+
+        @c.setter
+        def c(self, value):
+            self.a = value - self.b
+
+        @property
+        def d(self):
+            return self.a + self.b
+
+        @d.setter
+        def d(self, value):
+            self.a = value // 2
+            self.b = value - self.a
+
+    s = SampleClass()
+    a_m = Mock()
+    c_m = Mock()
+    d_m = Mock()
+    s.events.a.connect(a_m)
+    s.events.c.connect(c_m)
+    s.events.d.connect(d_m)
+
+    s.d = 5
+    a_m.assert_called_once()
+    c_m.assert_called_once()
+    d_m.assert_called_once()
