@@ -1,19 +1,22 @@
 import gc
+import os
 import sys
-import time
 from contextlib import suppress
 from functools import partial, wraps
 from inspect import Signature
-from typing import Optional
+from typing import Literal, Optional
 from unittest.mock import MagicMock, Mock, call
 
 import pytest
-import toolz
-from typing_extensions import Literal
 
-from psygnal import EmitLoopError, Signal, SignalInstance, _compiled
+import psygnal
+from psygnal import EmitLoopError, Signal, SignalInstance
 from psygnal._weak_callback import WeakCallback
 
+PY39 = sys.version_info[:2] == (3, 9)
+WINDOWS = os.name == "nt"
+COMPILED = psygnal._compiled
+
 
 def stupid_decorator(fun):
     def _fun(*args):
@@ -125,8 +128,7 @@ def test_decorator():
         raise err
 
     @emitter.one_int.connect(check_nargs=False)
-    def bad_cb(a, b, c):
-        ...
+    def bad_cb(a, b, c): ...
 
     with pytest.raises(EmitLoopError) as e:
         emitter.one_int.emit(1)
@@ -162,6 +164,7 @@ def test_signature_provided():
 
 def test_emit_checks():
     emitter = Emitter()
+    emitter.one_int.connect(f_no_arg)
 
     emitter.one_int.emit(check_nargs=False)
     emitter.one_int.emit()
@@ -256,9 +259,11 @@ def test_slot_types(type_: str) -> None:
     obj = MyObj()
 
     if type_ == "setattr":
-        signal.connect_setattr(obj, "x")
+        with pytest.warns(FutureWarning, match="The default value of maxargs"):
+            signal.connect_setattr(obj, "x")
     elif type_ == "setitem":
-        signal.connect_setitem(obj, "x")
+        with pytest.warns(FutureWarning, match="The default value of maxargs"):
+            signal.connect_setitem(obj, "x")
     elif type_ == "function":
         signal.connect(f_int)
 
@@ -269,8 +274,10 @@ def test_slot_types(type_: str) -> None:
     elif type_ == "partial_method":
         signal.connect(partial(obj.f_int_int, 2))
     elif type_ == "toolz_function":
+        toolz = pytest.importorskip("toolz")
         signal.connect(toolz.curry(f_int_int, 2))
     elif type_ == "toolz_method":
+        toolz = pytest.importorskip("toolz")
         signal.connect(toolz.curry(obj.f_int_int, 2))
     elif type_ == "partial_method_kwarg":
         signal.connect(partial(obj.f_int_int, b=2))
@@ -363,6 +370,7 @@ def test_weakref(slot):
     if slot == "partial":
         emitter.one_int.connect(partial(obj.f_int_int, 1))
     elif slot == "toolz_curry":
+        toolz = pytest.importorskip("toolz")
         emitter.one_int.connect(toolz.curry(obj.f_int_int, 1))
     else:
         emitter.one_int.connect(getattr(obj, slot))
@@ -385,35 +393,34 @@ def test_weakref(slot):
         "partial",
     ],
 )
-def test_group_weakref(slot):
+def test_group_weakref(slot) -> None:
     """Test that a connected method doesn't hold strong ref."""
     from psygnal import SignalGroup
 
     class MyGroup(SignalGroup):
         sig1 = Signal(int)
 
-    emitter = MyGroup()
+    group = MyGroup()
     obj = MyObj()
 
-    # simply by nature of being in a group, sig1 will have a callback
-    assert len(emitter.sig1) == 1
+    assert len(group.sig1) == 0
     # but the group itself doesn't have any
-    assert len(emitter) == 0
+    assert len(group._psygnal_relay) == 0
 
     # connecting something to the group adds to the group connections
-    emitter.connect(
+    group.all.connect(
         partial(obj.f_int_int, 1) if slot == "partial" else getattr(obj, slot)
     )
-    assert len(emitter.sig1) == 1
-    assert len(emitter) == 1
+    assert len(group.sig1) == 1
+    assert len(group._psygnal_relay) == 1
 
-    emitter.sig1.emit(1)
-    assert len(emitter.sig1) == 1
+    group.sig1.emit(1)
+    assert len(group.sig1) == 1
     del obj
     gc.collect()
-    emitter.sig1.emit(1)  # this should trigger deletion, so would emitter.emit()
-    assert len(emitter.sig1) == 1
-    assert len(emitter) == 0  # it's been cleaned up
+    group.sig1.emit(1)  # this should trigger deletion, so would emitter.emit()
+    assert len(group.sig1) == 0  # NOTE! this is 0 not 1, because the relay is also gone
+    assert len(group._psygnal_relay) == 0  # it's been cleaned up
 
 
 # def test_norm_slot():
@@ -553,15 +560,13 @@ def test_checking_off():
 
     # the no_check signal was instantiated with check_[nargs/types] = False
     @e.no_check.connect
-    def bad_in_many_ways(x: int, y, z):
-        ...
+    def bad_in_many_ways(x: int, y, z): ...
 
 
 def test_keyword_only_not_allowed():
     e = Emitter()
 
-    def f(a: int, *, b: int):
-        ...
+    def f(a: int, *, b: int): ...
 
     with pytest.raises(ValueError) as er:
         e.two_int.connect(f)
@@ -586,34 +591,6 @@ def test_unique_connections():
     assert len(e.one_int._slots) == 2
 
 
-@pytest.mark.skipif(_compiled, reason="passes, but segfaults on exit")
-def test_asynchronous_emit():
-    e = Emitter()
-    a = []
-
-    def slow_append(arg: int):
-        time.sleep(0.1)
-        a.append(arg)
-
-    mock = MagicMock(wraps=slow_append)
-    e.no_arg.connect(mock, unique=False)
-
-    assert not Signal.current_emitter()
-    value = 42
-    with pytest.warns(FutureWarning):
-        thread = e.no_arg.emit(value, asynchronous=True)
-    mock.assert_called_once()
-    assert Signal.current_emitter() is e.no_arg
-
-    # dude, you have to wait.
-    assert not a
-
-    if thread:
-        thread.join()
-    assert a == [value]
-    assert not Signal.current_emitter()
-
-
 def test_sig_unavailable():
     """In some cases, signature.inspect() fails on a callable, (many builtins).
 
@@ -717,9 +694,11 @@ def test_property_connect():
 
     a = A()
     emitter = Emitter()
-    emitter.one_int.connect_setattr(a, "x")
+
+    emitter.one_int.connect_setattr(a, "x", maxargs=1)
     assert len(emitter.one_int) == 1
-    emitter.two_int.connect_setattr(a, "x")
+    with pytest.warns(FutureWarning, match="The default value of maxargs"):
+        emitter.two_int.connect_setattr(a, "x")
     assert len(emitter.two_int) == 1
     emitter.one_int.emit(1)
     assert a.li == [1]
@@ -736,7 +715,7 @@ def test_property_connect():
     emitter.two_int.disconnect(s, missing_ok=False)
 
     with pytest.raises(AttributeError):
-        emitter.one_int.connect_setattr(a, "y")
+        emitter.one_int.connect_setattr(a, "y", maxargs=None)
 
 
 def test_connect_setitem():
@@ -752,7 +731,8 @@ def test_connect_setitem():
 
     t = T()
     my_obj = SupportsItem()
-    t.sig.connect_setitem(my_obj, "x")
+    with pytest.warns(FutureWarning, match="The default value of maxargs"):
+        t.sig.connect_setitem(my_obj, "x")
     t.sig.emit(5)
     assert my_obj._dict == {"x": 5}
     t.sig.disconnect_setitem(my_obj, "x")
@@ -761,7 +741,7 @@ def test_connect_setitem():
 
     obj = object()
     with pytest.raises(TypeError, match="does not support __setitem__"):
-        t.sig.connect_setitem(obj, "x")
+        t.sig.connect_setitem(obj, "x", maxargs=1)
 
     with pytest.raises(TypeError):
         t.sig.disconnect_setitem(obj, "x", missing_ok=False)
@@ -846,12 +826,7 @@ def test_emit_loop_exceptions():
         "f_any_assigned",
         "partial",
         "partial_kwargs",
-        pytest.param(
-            "partial",
-            marks=pytest.mark.xfail(
-                sys.version_info < (3, 8), reason="no idea why this fails on 3.7"
-            ),
-        ),
+        "partial",
     ],
 )
 def test_weakref_disconnect(slot):
@@ -1001,3 +976,98 @@ def test_pickle():
     x = pickle.loads(_dump)
     x.sig.emit()
     mock.assert_called_once()
+
+
+@pytest.mark.skipif(PY39 and WINDOWS and COMPILED, reason="fails")
+def test_recursion_error() -> None:
+    s = SignalInstance()
+
+    @s.connect
+    def callback() -> None:
+        s.emit()
+
+    with pytest.raises(RecursionError):
+        s.emit()
+
+
+def test_signal_order():
+    """Test that signals are emitted in the order they were connected."""
+    emitter = Emitter()
+    mock1 = Mock()
+    mock2 = Mock()
+
+    def callback(x):
+        if x != 10:
+            emitter.one_int.emit(10)
+
+    emitter.one_int.connect(mock1)
+    emitter.one_int.connect(callback)
+    emitter.one_int.connect(mock2)
+    emitter.one_int.emit(1)
+
+    mock1.assert_has_calls([call(1), call(10)])
+    mock2.assert_has_calls([call(1), call(10)])
+
+
+def test_signal_order_suspend():
+    """Test that signals are emitted in the order they were connected."""
+    emitter = Emitter()
+    mock1 = Mock()
+    mock2 = Mock()
+
+    def callback(x):
+        if x < 10:
+            emitter.one_int.emit(10)
+
+    def callback2(x):
+        if x == 10:
+            emitter.one_int.emit(11)
+
+    def callback3(x):
+        if x == 10:
+            with emitter.one_int.paused(reducer=lambda a, b: (a[0] + b[0],)):
+                for i in range(12, 15):
+                    emitter.one_int.emit(i)
+
+    emitter.one_int.connect(mock1)
+    emitter.one_int.connect(callback)
+    emitter.one_int.connect(callback2)
+    emitter.one_int.connect(callback3)
+    emitter.one_int.connect(mock2)
+    emitter.one_int.emit(1)
+
+    mock1.assert_has_calls([call(1), call(10), call(11), call(39)])
+    mock2.assert_has_calls([call(1), call(10), call(11), call(39)])
+
+
+def test_call_priority() -> None:
+    """Test that signals are emitted in the order they were connected."""
+    emitter = Emitter()
+
+    calls = []
+
+    emitter.no_arg.connect(lambda: calls.append(1))
+    emitter.no_arg.connect(lambda: calls.append(2), priority=5)
+    emitter.no_arg.connect(lambda: calls.append(3), priority=-5)
+    emitter.no_arg.connect(lambda: calls.append(4), priority=10)
+
+    emitter.no_arg.emit()
+    assert calls == [4, 2, 1, 3]
+
+
+def test_slotted_classes() -> None:
+    class T:
+        __slots__ = ("not_sig",)
+        sig = Signal()
+
+    t = T()
+    mock = Mock()
+
+    @t.sig.connect
+    def f():
+        mock()
+
+    t.sig.emit()
+    mock.assert_called_once()
+
+    assert t.sig is t.sig
